﻿// Copyright (c) 2020 Tudor Stanciu

using AutoMapper;
using MediatR;
using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using Tuitio.Application.Abstractions;
using Tuitio.Domain.Repositories;

namespace Tuitio.Application.Queries
{
    public class GetUserInfo
    {
        public class Query : IRequest<Model> { }

        public record Model
        {
            public int UserId { get; init; }
            public string UserName { get; init; }
            public string FirstName { get; init; }
            public string LastName { get; init; }
            public string Email { get; init; }
            public string ProfilePictureUrl { get; init; }
            public string SecurityStamp { get; init; }
            public DateTime CreationDate { get; init; }
            public int? FailedLoginAttempts { get; init; }
            public DateTime? LastLoginDate { get; init; }
            public Dictionary<string, string> Claims { get; init; }
            public UserRole[] UserRoles { get; init; }
            public UserGroup[] UserGroups { get; init; }
            public ContactOption[] ContactOptions { get; init; }
        }

        public record ContactOption
        {
            public int Id { get; init; }
            public string ContactTypeCode { get; init; }
            public string ContactTypeName { get; init; }
            public string ContactValue { get; init; }
        }

        public record UserGroup
        {
            public int Id { get; init; }
            public string Code { get; init; }
            public string Name { get; init; }
        }

        public record UserRole
        {
            public int Id { get; init; }
            public string Code { get; init; }
            public string Name { get; init; }
        }

        public class QueryHandler : IRequestHandler<Query, Model>
        {
            private readonly IUserRepository _userRepository;
            private readonly IHttpContextService _httpContextService;
            private readonly IMapper _mapper;

            public QueryHandler(IUserRepository userRepository, IHttpContextService httpContextService, IMapper mapper)
            {
                _userRepository=userRepository;
                _httpContextService=httpContextService;
                _mapper=mapper;
            }

            public async Task<Model> Handle(Query request, CancellationToken cancellationToken)
            {
                var userId = _httpContextService.GetUserId();
                var user = await _userRepository.GetFullUser(userId);
                var info = _mapper.Map<Model>(user);
                return info;
            }
        }
    }
}
